package de.thdeg.greiner.superpangworld.game.managers;

import de.thdeg.greiner.superpangworld.gameview.GameView;
import de.thdeg.greiner.superpangworld.graphics.base.Bubble;
import de.thdeg.greiner.superpangworld.graphics.base.CollidableGameObject;
import de.thdeg.greiner.superpangworld.graphics.base.Position;
import de.thdeg.greiner.superpangworld.graphics.moveable.Harpoon;
import de.thdeg.greiner.superpangworld.graphics.moveable.HexagonalBubble;
import de.thdeg.greiner.superpangworld.graphics.moveable.RoundBubble;
import de.thdeg.greiner.superpangworld.graphics.moveable.SpecialBubble;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

/**
 * The manager which handles the gameplay.
 */
public class GamePlayManager {

    /** The GameView on which the game is displayed */
    private GameView gameView;
    /** The manager, which handles the game objects */
    private GameObjectManager gameObjectManager;
    /** Flag, if the game objects were already deleted once */
    private boolean listHasBeenDeleted;
    /** Generator for random values */
    private Random random;

    /**
     * Create the manager handling the gameplay.
     * @param gameView
     * @param gameObjectManager
     */
    public GamePlayManager(GameView gameView, GameObjectManager gameObjectManager) {
        listHasBeenDeleted = false;
        this.gameView = gameView;
        this.gameObjectManager = gameObjectManager;
        this.random = new Random();
        gameObjectManager.getPlayer().setGamePlayManager(this);
    }

    /**
     * Handle the gameplay.
     */
    public void updateGamePlay(){
        spawnAndDestroyBubbles();
    }

    /**
     * Spawn and destory bubbles.
     */
    private void spawnAndDestroyBubbles(){
        // Spawn a bubble every second
        if(gameView.timerExpired("SpawnBubble","GamePlayManager")){
            addRandomBubble();
            gameView.setTimer("SpawnBubble","GamePlayManager",1000);
        }
        // Remove a bubble every 5 seconds
        if(!gameObjectManager.getBubbles().isEmpty() && gameView.timerExpired("DestroyBubble","GamePlayManager")){
            gameObjectManager.getBubbles().remove(random.nextInt(gameObjectManager.getBubbles().size()));
            gameView.setTimer("DestroyBubble","GamePlayManager",5000);
        }
        // Delete all bubbles after 10 seconds of playing
        if(!listHasBeenDeleted && gameView.getGameTimeInMilliseconds()>=10000){
            gameObjectManager.getBubbles().clear();
            listHasBeenDeleted = true;
        }


    }

    /**
     * Adds a random bubble to the game. Percentages:
     *   - 60% normal bubble
     *   - 30% hexagonal bubble
     *   - 10% special bubble
     */
    private void addRandomBubble(){
        int randomNumber = random.nextInt(100);
        if(randomNumber<60){
            RoundBubble roundBubble = new RoundBubble(gameView, new ArrayList<>(Arrays.asList(gameObjectManager.getPlayer())));
            roundBubble.setGamePlayManager(this);
            gameObjectManager.getBubbles().add(roundBubble);
        }else if(randomNumber<90){
            HexagonalBubble hexagonalBubble = new HexagonalBubble(gameView, new ArrayList<>(Arrays.asList(gameObjectManager.getPlayer())));
            hexagonalBubble.setGamePlayManager(this);
            gameObjectManager.getBubbles().add(hexagonalBubble);
        }else{
            SpecialBubble specialBubble = new SpecialBubble(gameView, new ArrayList<>(Arrays.asList(gameObjectManager.getPlayer())));
            specialBubble.setGamePlayManager(this);
            gameObjectManager.getBubbles().add(specialBubble);
        }
    }

    /**
     * Shoot a harpoon starting at a given position.
     * @param startPosition the start position
     */
    public void shootHarpoon(Position startPosition){
        if(gameView.timerExpired("ShootHarpoon","GamePlayManager")){
            ArrayList<CollidableGameObject> collidableGameObjects = new ArrayList<>();
            collidableGameObjects.addAll(gameObjectManager.getBubbles());
            Harpoon harpoon = new Harpoon(gameView,collidableGameObjects);
            harpoon.getPosition().setTo(startPosition.x, startPosition.y);
            harpoon.setGamePlayManager(this);
            gameObjectManager.getHarpoons().add(harpoon);
            gameView.setTimer("ShootHarpoon","GamePlayManager",300);
        }
    }

    /**
     * Remove a harpoon from the game.
     * @param harpoon the harpoon to remove
     */
    public void destroy(Harpoon harpoon){
        gameObjectManager.getHarpoons().remove(harpoon);
    }
    /**
     * Remove a bubble from the game.
     * @param bubble the bubble to remove
     */
    public void destroy(Bubble bubble){
        gameObjectManager.getBubbles().remove(bubble);
    }

}
